<!-----------------------------------------------------------------------
Author 	 :	Luis Majano
Date     :	September 25, 2005
Description :

	The galleon administrator port.

----------------------------------------------------------------------->
<cfcomponent name="ehForums" extends="coldbox.system.eventhandler" output="false">

	<!--- ************************************************************* --->
	<cffunction name="onAppStart" access="public" returntype="void" output="false">
		<!--- App Init Code --->
		<cfset structDelete(application, "userCache")>
		<!--- Get main settings --->
		<cfinvoke component="#getSetting("ParentMapping")#/cfcs/galleon" method="getSettings" returnVariable="settings">
		<cfset application.settings = settings>
		<!--- get user CFC --->
		<cfset application.user = createObject("component","#getSetting("ParentMapping")#.cfcs.user").init(application.settings)>
		<!--- get utils CFC --->
		<cfset application.utils = createObject("component","#getSetting("ParentMapping")#.cfcs.utils")>
		<!--- get conference CFC --->
		<cfset application.conference = createObject("component","#getSetting("ParentMapping")#.cfcs.conference").init(application.settings)>
		<!--- get forum CFC --->
		<cfset application.forum = createObject("component","#getSetting("ParentMapping")#.cfcs.forum").init(application.settings)>
		<!--- get thread CFC --->
		<cfset application.thread = createObject("component","#getSetting("ParentMapping")#.cfcs.thread").init(application.settings)>
		<!--- get message CFC --->
		<cfset application.message = createObject("component","#getSetting("ParentMapping")#.cfcs.message").init(application.settings)>
		<!--- get rank CFC --->
		<cfset application.rank = createObject("component","#getSetting("ParentMapping")#.cfcs.rank").init(application.settings)>
		<cfset application.init = true>
	</cffunction>
	<!--- ************************************************************* --->
	
	<!--- ************************************************************* --->
	<cffunction name="onRequestStart" access="public" returntype="void" output="false">

		<!--- Logout Code --->
		<cfif valueExists("logout")>
			<cfset doLogout()>
		</cfif>

		<!--- handle security --->
		<cflogin></cflogin>

		<!--- Used by index, forums, and threads ---->
		<cfset paramValue("sort", "")>

		<cfif not request.udf.isLoggedOn() and not findnocase("doLogin", getValue("event"))>
			<cfset setValue("event","ehForums.dspLogin")>
		</cfif>

		<!--- must be the correct authentication --->
		<cfif not isUserInRole("forumsadmin") and not findnocase("Login", getValue("event"))>
			<cflocation url="../" addtoken="false">
			<cfabort>
		</cfif>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehConferences = "ehforums.dspConferences">
		<cfset rc.xehForums = "ehforums.dspForums">
		<cfset rc.xehThreads = "ehforums.dspThreads">
		<cfset rc.xehMessages = "ehforums.dspMessages">
		<cfset rc.xehRanks = "ehforums.dspRanks">
		<cfset rc.xehUsers = "ehforums.dspUsers">
		<cfset rc.xehStats = "ehforums.dspStats">
		<cfset rc.xehSearchStats = "ehForums.dspSearchStats">
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspHome" access="public" returntype="void" output="false">
		<!--- Set Title and Templatename --->
		<cfset setValue("title","Welcome to the Galleon Administrator")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwIndex")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- SECURITY HANDLERS --->
	<!--- ************************************************************* --->
	<cffunction name="dspLogin" access="public" returntype="void" output="false">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehLogin = "ehForums.doLogin">
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwLogin")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doLogin" access="public" returntype="void" output="false">
		<cfset var mygroups = "">
		<!--- handle security --->
		<cflogin>
			<cfif application.user.authenticate(trim(getValue("username")), trim(getValue("password")))>
				<!--- good logon, grab their groups --->
				<cfset mygroups = application.user.getGroupsForUser(trim(getValue("username")))>
				<cfset session.user = application.user.getUser(trim(getValue("username")))>
				<cfloginuser name="#trim(getValue("username"))#" password="#trim(getValue("password"))#" roles="#mygroups#">
			<cfelse>
				<cfset getPlugin("messagebox").setMessage("error","Your username and password did not work.")>
				<cfset setNextEvent("ehForums.dspLogin","failedLogon=true&ref=#getValue("ref","")#")>
			</cfif>
		</cflogin>
		<!-- Set NExt Event --->
		<cfset setNextEvent("ehForums.dspHome")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doLogout" access="public" returntype="void" output="false">
		<!--- handle security --->
		<cflogout>
		<cfset setNextEvent("ehForums.dspLogin")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- CONFERENCES --->
	<!--- ************************************************************* --->
	<cffunction name="dspConferences" access="public" returntype="void" output="false">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehConferencesEdit = "ehForums.dspConferencesEdit">
		
		<!--- handle deletions --->
		<cfif valueExists("mark") and len(getValue("mark"))>
			<cfset doDeleteConference()>
		</cfif>

		<!--- get conferences --->
		<cfset setValue("conferences", application.conference.getConferences(false))>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Conference Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwConferences")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteConference" access="public" returntype="void" output="false">
		<cfset var id = "">
		<cfloop index="id" list="#getValue("mark")#">
			<cfset application.conference.deleteConference(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Conferences deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspConferencesEdit" access="public" returntype="void" output="false">
		<cfset var conference = "">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehSaveConference = "ehForums.doSaveConference">
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspConferences")>
		</cfif>

		<cfif getValue("id") neq "0">
			<cfset conference = application.conference.getConference(getValue("id")) >
			<cfset setValue("name", conference.name)>
			<cfset setValue("description", conference.description)>
			<cfset setValue("active", conference.active)>
		<cfelse>
			<cfset setValue("active", false)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Conference Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwConferences_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveConference" access="public" returntype="void" output="false">
		<cfset var errors = "">
		<cfset var conference = "">
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspConferences")>
		</cfif>

		<cfif not len(trim(getValue("name")))>
			<cfset errors = errors & "You must specify a name.<br>">
		</cfif>
		<cfif not len(trim(getValue("description")))>
			<cfset errors = errors & "You must specify a description.<br>">
		</cfif>
		<cfif not len(errors)>
			<cfset conference = structNew()>
			<cfset conference.name = trim(htmlEditFormat(getValue("name")))>
			<cfset conference.description = trim(htmlEditFormat(getValue("description")))>
			<cfset conference.active = trim(htmlEditFormat(getValue("active")))>
			<cfif getValue("id") neq 0>
				<cfset application.conference.saveConference(getValue("id"), conference)>
			<cfelse>
				<cfset application.conference.addConference(conference)>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Conferfence, #conference.name#, has been updated.")>
			<cfset setNextEvent("ehForums.dspConferences")>
		</cfif>
		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspConferencesEdit","id=#getValue("id")#")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- FORUMS --->
	<!--- ************************************************************* --->
	<cffunction name="dspForums" access="public" returntype="void" output="false">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehForumsEdit = "ehForums.dspForumsEdit">
		<!--- handle deletions --->
		<cfif valueExists("mark") and len(getValue("mark"))>
			<cfset doDeleteForums()>
		</cfif>

		<!--- get forums --->
		<cfset setValue("forums",application.forum.getForums(false) )>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Forum Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwForums")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteForums" access="public" returntype="void" output="false">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#getValue("mark")#">
			<cfset application.forum.deleteForum(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Forums deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspForumsEdit" access="public" returntype="void" output="false">
		<cfset var forum = "">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehForumsSave = "ehForums.doSaveForums">
		
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspForums")>
		</cfif>

		<!--- get all conferences --->
		<cfset setValue("conferences", application.conference.getConferences(false))>

		<cfif getValue("id") neq "0">
			<cfset forum = application.forum.getForum(getValue("id"))>
			<cfset setValue("name", forum.name)>
			<cfset setValue("description", forum.description)>
			<cfset setValue("readonly", forum.readonly)>
			<cfset setValue("active", forum.active)>
			<cfset setValue("conferenceidfk", forum.conferenceidfk)>
		<cfelse>
			<cfset setValue("readonly",false)>
			<cfset setValue("active", false)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Conference Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwForums_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveForums" access="public" returntype="void" output="false">
		<cfset var errors = "">
		<cfset var forum = "">
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspForums")>
		</cfif>
		
		<cfif not len(trim(getValue("name")))>
			<cfset errors = errors & "You must specify a name.<br>">
		</cfif>
		<cfif not len(trim(getValue("description")))>
			<cfset errors = errors & "You must specify a description.<br>">
		</cfif>
		<cfif not len(errors)>
			<cfset forum = structNew()>
			<cfset forum.name = trim(htmlEditFormat(getValue("name")))>
			<cfset forum.description = trim(htmlEditFormat(getValue("description")))>
			<cfset forum.readonly = trim(htmlEditFormat(getValue("readonly")))>
			<cfset forum.active = trim(htmlEditFormat(getValue("active")))>
			<cfset forum.conferenceidfk = trim(htmlEditFormat(getValue("conferenceidfk")))>

			<cfif getValue("id") neq 0>
				<cfset application.forum.saveForum(getValue("id"), forum)>
			<cfelse>
				<cfset application.forum.addForum(forum)>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Forum, #forum.name#, has been updated.")>
			<cfset setNextEvent("ehForums.dspForums")>
		</cfif>

		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspForumsEdit","id=#getValue("id")#")>
	</cffunction>
	<!--- ************************************************************* --->


	<!--- THREADS --->
	<!--- ************************************************************* --->
	<cffunction name="dspThreads" access="public" returntype="void" output="false">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehThreadsEdit = "ehForums.dspThreadsEdit">
		
		<!--- handle deletions --->
		<cfif valueExists("mark") and len(getValue("mark"))>
			<cfset doDeleteThreads()>
		</cfif>

		<!--- get Threads --->
		<cfset setValue("threads",application.thread.getThreads(false) )>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Thread Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwThreads")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteThreads" access="public" returntype="void" output="false">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#getValue("mark")#">
			<cfset application.thread.deleteThread(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Threads deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspThreadsEdit" access="public" returntype="void" output="false">
		<cfset var thread = "">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehThreadsSave = "ehForums.doSaveThreads">
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspThreads")>
		</cfif>

		<!--- get all forums --->
		<cfset setValue("forums", application.forum.getForums(false) )>
		<!--- get all users --->
		<cfset setValue("users", application.user.getUsers() )>

		<cfif getValue("id") neq "0">
			<cfset thread = application.thread.getThread(getValue("id"))>
			<cfset setValue("name", thread.name)>
			<cfset setValue("readonly", thread.readonly)>
			<cfset setValue("active", thread.active)>
			<cfset setValue("forumidfk", thread.forumidfk)>
			<cfset setValue("datecreated", dateFormat(thread.datecreated,"m/dd/yy"))>
			<cfset setValue("useridfk", thread.useridfk)>
			<cfset setValue("sticky", thread.sticky)>
		<cfelse>
			<cfset setValue("readonly", false)>
			<cfset setValue("active", false)>
			<cfset setValue("datecreated", dateFormat(now(),"m/dd/yy"))>
			<cfset setValue("sticky", false)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Conference Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwThreads_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveThreads" access="public" returntype="void" output="false">
		<cfset var errors = "">
		<cfset var thread = "">
		
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspThreads")>
		</cfif>
		
		<cfif not len(trim(getValue("name")))>
			<cfset errors = errors & "You must specify a name.<br>">
		</cfif>
		<cfif not len(trim(getValue("datecreated"))) or not isDate(getValue("datecreated"))>
			<cfset errors = errors & "You must specify a valid creation date.<br>">
		</cfif>

		<cfif not len(errors)>
			<cfset thread = structNew()>
			<cfset thread.name = trim(htmlEditFormat(getValue("name")))>
			<cfset thread.readonly = trim(htmlEditFormat(getValue("readonly")))>
			<cfset thread.active = trim(htmlEditFormat(getValue("active")))>
			<cfset thread.forumidfk = trim(htmlEditFormat(getValue("forumidfk")))>
			<cfset thread.datecreated = trim(htmlEditFormat(getValue("datecreated")))>
			<cfset thread.useridfk = trim(htmlEditFormat(getValue("useridfk")))>
			<cfset thread.sticky = trim(htmlEditFormat(getValue("sticky")))>
			<cfif getValue("id") neq 0>
				<cfset application.thread.saveThread(getValue("id"), thread)>
			<cfelse>
				<cfset application.thread.addThread(thread)>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Thread, #thread.name#, has been updated.")>
			<cfset setNextEvent("ehForums.dspThreads")>
		</cfif>

		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspThreadsEdit","id=#getValue("id")#")>
	</cffunction>
	<!--- ************************************************************* --->


	<!--- MESSAGES --->
	<!--- ************************************************************* --->
	<cffunction name="dspMessages" access="public" returntype="void" output="false">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehMessagesEdit = "ehForums.dspMessagesEdit">
		<!--- handle deletions --->
		<cfif valueExists("mark") and len(getValue("mark"))>
			<cfset doDeleteMessages()>
		</cfif>

		<!--- get Threads --->
		<cfset setValue("messages",application.message.getMessages() )>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Messages Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwMessages")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteMessages" access="public" returntype="void" output="false">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#getValue("mark")#">
			<cfset application.message.deleteMessage(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Messages deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspMessagesEdit" access="public" returntype="void" output="false">
		<cfset var message = "">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehMessagesSave = "ehForums.doSaveMessages">
		
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspMessages")>
		</cfif>

		<!--- get all forums --->
		<cfset setValue("threads", application.thread.getThreads(false) )>
		<!--- get all users --->
		<cfset setValue("users", application.user.getUsers() )>

		<cfif getValue("id") neq "0">
			<cfset message = application.message.getMessage(getValue("id"))>
			<cfset setValue("title", message.title)>
			<cfset setValue("body", message.body)>
			<cfset setValue("posted", dateFormat(message.posted,"m/dd/yy") & timeFormat(message.posted,"h:mm tt"))>
			<cfset setValue("useridfk", message.useridfk)>
			<cfset setValue("threadidfk", message.threadidfk)>
		<cfelse>
			<cfset setValue("posted","#dateFormat(now(),"m/dd/yy")# #timeFormat(now(),"h:mm tt")#")>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Conference Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwMessages_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveMessages" access="public" returntype="void" output="false">
		<cfset var errors = "">
		<cfset var message = "">
		<cfset var users = "">
		<cfset var threadPicked = "">
		<cfset var theusername = "">
		
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspMessages")>
		</cfif>

		<cfif not len(trim(getValue("title")))>
			<cfset errors = errors & "You must specify a title.<br>">
		</cfif>
		<cfif not len(trim(getValue("body")))>
			<cfset errors = errors & "You must specify a body.<br>">
		</cfif>
		<cfif not len(trim(getValue("posted"))) or not isDate(getValue("posted"))>
			<cfset errors = errors & "You must specify a valid creation date.<br>">
		</cfif>

		<cfif not len(errors)>
			<cfset message = structNew()>
			<cfset message.title = trim(htmlEditFormat(getValue("title")))>
			<cfset message.body = trim(htmlEditFormat(getValue("body")))>
			<cfset message.posted = trim(getValue("posted"))>
			<cfset message.threadidfk = getValue("threadidfk")>
			<cfset message.useridfk = getValue("useridfk")>

			<!--- get all users --->
			<cfset users = application.user.getUsers() >

			<cfif getValue("id") neq 0>
				<cfset application.message.saveMessage(getValue("id"), message)>
			<cfelse>
				<cfset threadPicked = application.thread.getThread(getValue("threadidfk"))>
				<!--- translate the user id to the username for addMessage --->
				<cfloop query="users">
					<cfif getvalue("id") is getValue("useridfk")>
						<cfset theusername = username>
					</cfif>
				</cfloop>
				<cfset application.message.addMessage(message,threadPicked.forumidfk,theusername,getValue("threadidfk"))>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Message, #message.title#, has been updated.")>
			<cfset setNextEvent("ehForums.dspMessages")>
		</cfif>
		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspMessagesEdit","id=#getValue("id")#")>
	</cffunction>
	<!--- ************************************************************* --->


	<!--- RANKS --->
	<!--- ************************************************************* --->
	<cffunction name="dspRanks" access="public" returntype="void" output="false">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehRanksEdits = "ehForums.dspRanksEdit">
		<!--- handle deletions --->
		<cfif valueExists("mark") and len(getValue("mark"))>
			<cfset doDeleteRanks()>
		</cfif>

		<!--- get Ranks --->
		<cfset setValue("ranks",application.rank.getRanks() )>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Rank Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwRanks")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteRanks" access="public" returntype="void" output="false">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#getValue("mark")#">
			<cfset application.rank.deleteRank(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Ranks deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspRanksEdit" access="public" returntype="void" output="false">
		<cfset var rank = "">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehRanksSave = "ehForums.doSaveRanks">
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspRanks")>
		</cfif>

		<cfif getValue("id") neq "0">
			<cfset rank = application.rank.getRank(getValue("id"))>
			<cfset setValue("name", rank.name)>
			<cfset setValue("minposts", rank.minposts)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Conference Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwRanks_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveRanks" access="public" returntype="void" output="false">
		<cfset var errors = "">
		<cfset var rank = "">
		
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspRanks")>
		</cfif>

		<cfif not len(trim(getValue("name")))>
			<cfset errors = errors & "You must specify a name.<br>">
		</cfif>
		<cfif not len(trim(getValue("minposts"))) or not isNumeric(getValue("minposts")) or getValue("minposts") lt 0>
			<cfset errors = errors & "You must specify a numeric value greater than zero for the minimum number of posts.<br>">
		</cfif>
		<cfif not len(errors)>
			<cfset rank = structNew()>
			<cfset rank.name = trim(htmlEditFormat(getValue("name")))>
			<cfset rank.minposts = trim(htmlEditFormat(getValue("minposts")))>
			<cfif getValue("id") neq 0>
				<cfset application.rank.saveRank(getValue("id"), rank)>
			<cfelse>
				<cfset application.rank.addRank(rank)>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Rank, #rank.name#, has been updated.")>
			<cfset setNextEvent("ehForums.dspRanks")>
		</cfif>

		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspRanksEdit","id=#getValue("id")#")>

	</cffunction>
	<!--- ************************************************************* --->

	<!--- USERS --->
	<!--- ************************************************************* --->
	<cffunction name="dspUsers" access="public" returntype="void" output="false">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehUsersEdit = "ehForums.dspUsersEdit">
		<!--- handle deletions --->
		<cfif valueExists("mark") and len(getValue("mark"))>
			<cfset doDeleteUsers()>
		</cfif>

		<!--- get Ranks --->
		<cfset setValue("users",application.user.getUsers() )>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Users Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwUsers")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteUsers" access="public" returntype="void" output="false">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#getValue("mark")#">
			<cfset application.user.deleteUser(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Users deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspUsersEdit" access="public" returntype="void" output="false">
		<cfset var user = "">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehUsersSave = "ehForums.doSaveUsers">
		
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspUsers")>
		</cfif>

		<cfset setValue("qGroups",application.user.getGroups())>

		<cfif getValue("id") gte "1">
			<cfset user = application.user.getUser(getValue("id"))>
			<cfset setValue("username", user.username)>
			<cfset setValue("emailaddress", user.emailaddress)>
			<cfset setValue("password", user.password)>
			<cfset setValue("datecreated", dateFormat(user.datecreated,"m/dd/yy") & timeFormat(user.datecreated,"h:mm tt"))>
			<cfset setValue("groups", user.groups)>
		<cfelse>
			<cfset setValue("datecreated",dateFormat(now(),"m/dd/yy") & timeFormat(now(),"h:mm tt") )>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Conference Editor")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwUsers_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveUsers" access="public" returntype="void" output="false">
		<cfset var errors = "">
		<cfset var user = "">
		
		<!--- Checks --->
		<cfif valueExists("cancel") or not valueExists("id") or not len(getValue("id"))>
			<cfset setNextEvent("ehForums.dspUsers")>
		</cfif>
		
		<cfif not len(trim(getValue("username")))>
			<cfset errors = errors & "You must specify a username.<br>">
		</cfif>
		<cfif not len(trim(getValue("emailaddress"))) or not request.udf.isEmail(trim(getValue("emailaddress")))>
			<cfset errors = errors & "You must specify an emailaddress.<br>">
		</cfif>
		<cfif not len(trim(getValue("password")))>
			<cfset errors = errors & "You must specify a password.<br>">
		</cfif>
		<cfif not len(trim(getValue("datecreated"))) or not isDate(getValue("datecreated"))>
			<cfset errors = errors & "You must specify a creation date.<br>">
		</cfif>
		<cfif not len(errors)>
			<cfset user = structNew()>
			<cfset setValue("username", trim(htmlEditFormat(getValue("username"))))>
			<cfset setValue("emailaddress", trim(htmlEditFormat(getValue("emailaddress"))))>
			<cfset setValue("password",trim(htmlEditFormat(getValue("password"))))>
			<cfset setValue("datecreated", trim(htmlEditFormat(getValue("datecreated"))))>
			<cfif getvalue("id") neq 0>
				<cfset application.user.saveUser(getValue("username"), getValue("password"), getValue("emailaddress"), getValue("datecreated"),getValue("groups"))>
			<cfelse>
				<cftry>
					<cfset application.user.addUser(getValue("username"), getValue("password"), getValue("emailaddress"), getValue("groups"))>
					<cfcatch>
						<cfset errors = cfcatch.message>
					</cfcatch>
				</cftry>
			</cfif>
			<cfif not len(errors)>
				<!--- MessageBox --->
				<cfset getPlugin("messagebox").setMessage("info","User, #getValue("username")#, has been updated.")>
				<cfset setNextEvent("ehForums.dspUsers")>
			</cfif>
		</cfif>
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspUsersEdit","id=#getValue("id")#")>

	</cffunction>
	<!--- ************************************************************* --->


	<!--- STATS --->
	<!--- ************************************************************* --->
	<cffunction name="dspStats" access="public" returntype="void" output="false">

		<cfset setValue("charts", true)>
		<cfif server.coldfusion.productname is "BlueDragon">
			<cfset setValue("charts", false)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset setValue("title","Galleon Stats")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwStats")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doGetStats" access="public" returntype="void" output="false">
		<cfset var messages = "">
		<!--- The framework has the ability to call and run an event handler from a tag --->
		<cfset setValue("conferences", application.conference.getConferences())>
		<cfset setValue("forums",application.forum.getForums())>
		<cfset setValue("threads", application.thread.getThreads())>
		<cfset setValue("users", application.user.getUsers())>
		<cfset messages = application.message.getMessages()>

		<!--- get first post --->
		<cfquery name="getMinPost" dbtype="query">
		select 	min(posted) as earliestPost
		from	messages
		</cfquery>
		<!--- get last post --->
		<cfquery name="getMaxPost" dbtype="query">
		select 	max(posted) as lastPost
		from	messages
		</cfquery>

		<!--- Save in Request --->
		<cfset setValue("getMinPost", getMinPost)>
		<cfset setValue("getMaxPost", getMaxPost)>
		<cfset setValue("messages",messages)>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspSearchStats" access="public" returntype="void" output="false">
		<cfset var prefix = application.settings.tableprefix>
		<!--- will be used to generate general stats - so we get all from the last 365 days --->
		<cfset var oldest = dateAdd("d", -365, now())>
		<cfset var searchstats = "">
		<cfset var today = createDateTime(year(now()), month(now()), day(now()), 0,0,0)>
		<cfset var past30 = dateAdd("d", -30, now())>
		<cfset var past365 = dateAdd("d", -365, now())>
		
		<cfquery datasource="#application.settings.dsn#" name="searchstats">
			select	searchterms, datesearched
			from	#prefix#search_log
			where	datesearched > <cfqueryparam cfsqltype="cf_sql_date" value="#oldest#">
		</cfquery>
		
		<!--- ineffecient - but gets around the lack of a limit op in ms access --->
		<cfquery name="rc.latest" dbtype="query" maxrows="1">
			select searchterms, datesearched
			from	searchstats
			order by datesearched desc
		</cfquery>
		
		<cfquery name="rc.todaysbest" dbtype="query" maxrows="1">
			select	searchterms, count(searchterms) as score
			from	searchstats
			where	datesearched > <cfqueryparam cfsqltype="cf_sql_timestamp" value="#today#">
			group by searchterms
			order by score desc
		</cfquery>
		
		
		<cfquery name="rc.thismonth" dbtype="query" maxrows="1">
			select	searchterms, count(searchterms) as score
			from	searchstats
			where	datesearched > <cfqueryparam cfsqltype="cf_sql_timestamp" value="#past30#">
			group by searchterms
			order by score desc
		</cfquery>
		
		<cfquery name="rc.thisyear" dbtype="query" maxrows="1">
			select	searchterms, count(searchterms) as score
			from	searchstats
			where	datesearched > <cfqueryparam cfsqltype="cf_sql_timestamp" value="#past365#">
			group by searchterms
			order by score desc
		</cfquery>

		<cfquery name="rc.top30" dbtype="query" maxrows="30">
			select	searchterms, count(searchterms) as score
			from	searchstats
			group by searchterms
			order by score desc
		</cfquery>


		<!--- Set Title and Templatename --->
		<cfset setValue("title","Galleon Search Stats")>
		<cfset setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset setView("vwSearch_stats")>
	</cffunction>
	<!--- ************************************************************* --->


	</cfcomponent>