<!-----------------------------------------------------------------------
Copyright 2005 - 2006 ColdBox Framework by Luis Majano
www.coldboxframework.com | www.coldboxframework.org
-------------------------------------------------------------------------

Author: Oscar Arevalo (oarevalo@gmail.com) & Luis Majano
Date:   October/2005
Description: 

This tool is used to display information about all components located within
the same directory. The tools is based on the coldfusion.runtime.TemplateProxy
java object to obtain this information. The tool displays information
regarding comments, methods, return types, and arguments.
Additionally, this tool scans each component for specially formatted TODO comments
to allow developers to keep track of pending tasks when developing their components.

This plugin only works in CFMX 6.0 and above. Not in Blue Dragon.

Modifications:
07/29/2006 - Added output=false to missing methods.  Var scope checking. And hint addition.
08/20/2006 - Modified for v1.1.0/ Added the renderer and stylesheet. It now becomes a full fledged documenter.
---------------------------------------------->
<cfcomponent displayname="cfcViewer" hint="This components provides functionality to obtain information about cfcs via introspection." extends="coldbox.system.plugin">

<!------------------------------------------- CONSTRUCTOR ------------------------------------------->
	
	<!--- ************************************************************* --->
	<cffunction name="init" access="public" returntype="any" output="false">
		<cfset super.Init() />
		<cfset variables.instance.pluginName = "CFC Viewer">
		<cfset variables.instance.pluginVersion = "1.0">
		<cfset variables.instance.pluginDescription = "CFC metadata introspection plugin">
		<!--- Plugin properties --->
		<cfset variables.instance.dirpath = "">
		<cfset variables.instance.cfcpath = "">
		<cfset variables.instance.aCFC = ArrayNew(1)>
		<cfset variables.instance.aPacks = ArrayNew(1)>
		<cfset variables.instance.styleSheet = "/coldbox/system/includes/cfcviewer.css">
		<cfset variables.instance.lstAccessTypes = "public,private,remote,package">
		<cfreturn this>
	</cffunction>
	<!--- ************************************************************* --->

<!------------------------------------------- PUBLIC ------------------------------------------->

	<!--- ************************************************************* --->
	<cffunction name="setup" access="public" output="false" returntype="Any"
				hint="This function takes care of component initialization.
						The function takes a relative path as a parameter, and looks for all cfcs within
						that path. Also, keeps track of all subdirectories and special TO DO notes
						within each component. Use the get* methods to access this data.">
		<!--- ************************************************************* --->
		<cfargument name="dirpath" type="string" required="yes" hint="Directory path to inspect for cfc's, not expanded, relative path. This path will be expanded.">
		<cfargument name="cfcpath" type="string" required="yes" hint="CFMapping path or full to cfc's. The path if you needed to instantiate the cfc.">
		<cfargument name="accessTypesFilter" type="string" required="no" default=""
					hint="Use this argument to only display methods with the access types given. If empty, displays all methods (public, private, remote).">
		<!--- ************************************************************* --->
		<cfset var qryCFC = "">
		<cfset var i = 0>
		<cfset var tmpName = "">
		<cfset var tmpPath = "">
		<cfset var iStartPos = 0>
		<cfset var iEndPos = 0>
		<cfset var txtFile = "">
		<cfset instance.aCFC = ArrayNew(1)>
		<cfset instance.aPacks = ArrayNew(1)>
		<cfset instance.dirpath = arguments.dirpath>
		<cfset instance.cfcpath = arguments.cfcpath>
		
		<cfif arguments.accessTypesFilter neq "">
			<cfset variables.instance.lstAccessTypes = arguments.accessTypesFilter>
		</cfif>
		
		<!--- get the list of cfc files on that directory --->
		<cfdirectory action="list" directory="#ExpandPath(instance.dirpath)#" name="qryCFC" sort="name">
		<cfset instance.qryCFC = qryCFC>

		<cfscript>
			// put components into arrays
			for(i=1;i lte qryCFC.RecordCount;i=i+1) {
				if(Right(qryCFC.Name[i],4) eq ".cfc" and Left(qryCFC.Name[i],1) neq ".") {
					tmpName = ListGetAt(qryCFC.Name[i],1,".");
					ArrayAppend(instance.aCFC, tmpName);
				}
				if(qryCFC.Type[i] eq "dir")
					ArrayAppend(instance.aPacks, qryCFC.Name[i]);
			}
			
			// add a slash at the end of the path (needed later)
			if(Right(instance.dirpath,1) neq "/") 
				instance.dirpath = instance.dirpath & "/";
		//Return instance
		return this;		
		</cfscript>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="getCFCs" access="public" returntype="array" output="false"
				hint="returns an array with the names of all components within the current directory">
		<cfreturn instance.aCFC>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="getPKGs" access="public" returntype="array" output="false"
				hint="returns an array with the names of all subdirectories (or packages) in the current directory">
		<cfreturn instance.aPacks>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="getCFCMetaData" access="public" returntype="any" output="false"
				hint="returns a structure with information about the given component. This structure contains
					information about returntype, methods, parameters, etc.">
		<!--- ************************************************************* --->
		<cfargument name="cfcName" type="string" required="yes">
		<!--- ************************************************************* --->
		<cfset var proxy = CreateObject("java", "coldfusion.runtime.TemplateProxy")>
		<cfset var cfcPathDot = ListChangeDelims(instance.cfcpath,".","/")>
		<cfset var cfcloc = cfcPathDot & "." & Arguments.cfcName>
		<cfset var md = "">
		<cftry>
			<cfset md = proxy.getMetaData(cfcloc, getPageContext())>
			<cfcatch  type="any">
				<cfthrow type="Framework.plugins.cfcViewer.GettingMetaDataException" message="#cfcatch.Message#" detail="#cfcatch.Detail#">
			</cfcatch>
		</cftry>
		<cfreturn md>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="render" access="public" returntype="any" output="false">
		<cfset var ccv = "">
		<cfsavecontent variable="ccv"><cfinclude template="../includes/cfcviewer.cfm"></cfsavecontent>
		<cfreturn ccv>
	</cffunction>
	<!--- ************************************************************* --->

<!------------------------------------------- INTANCE ACCESSORS/MUTATORS ------------------------------------------->

	<!--- ************************************************************* --->
	<cffunction name="getStyleSheet" access="public" returntype="string" output="false"	hint="Get the stylesheet to use when rendering the documentation.">
		<cfreturn instance.styleSheet>
	</cffunction>
	<!--- ************************************************************* --->
	
	<!--- ************************************************************* --->
	<cffunction name="setStyleSheet" access="public" returntype="void" output="false" hint="Set the stylesheet to use when rendering the documentation">
		<cfargument name="styleSheet" type="string" required="true">
		<cfset instance.styleSheet = arguments.styleSheet>
	</cffunction>
	<!--- ************************************************************* --->
	
	<!--- ************************************************************* --->
	<cffunction name="getDirpath" access="public" returntype="string" output="false" hint="Get the dirpath of where the cfc's reside. This is expanded.">
		<cfreturn instance.dirpath>
	</cffunction>
	<!--- ************************************************************* --->
	
	<!--- ************************************************************* --->
	<cffunction name="setdirpath" access="public" returntype="void" output="false" hint="Set the dirpath">
		<cfargument name="dirpath" type="string" required="true">
		<cfset instance.dirpath = arguments.dirpath>
	</cffunction>
	<!--- ************************************************************* --->
	
	<!--- ************************************************************* --->
	<cffunction name="getcfcpath" access="public" returntype="string" output="false" hint="Get cfc invocation path">
		<cfreturn instance.cfcpath>
	</cffunction>
	<!--- ************************************************************* --->
	
	<!--- ************************************************************* --->
	<cffunction name="setcfcpath" access="public" returntype="string" output="false" hint="Set the cfc invocation path">
		<cfargument name="cfcpath" type="string" required="true">
		<cfset instance.cfcpath = arguments.cfcpath>
	</cffunction>
	<!--- ************************************************************* --->
	
	<!--- ************************************************************* --->
	<cffunction name="getlstAccessTypes" access="public" returntype="string" output="false" hint="Get lstAccessTypes">
		<cfreturn instance.lstAccessTypes>
	</cffunction>
	<!--- ************************************************************* --->
	
	<!--- ************************************************************* --->
	<cffunction name="setlstAccessTypes" access="public" returntype="void" output="false" hint="Set lstAccessTypes">
		<cfargument name="lstAccessTypes" type="string" required="true">
		<cfset instance.lstAccessTypes = arguments.lstAccessTypes>
	</cffunction>
	<!--- ************************************************************* --->

</cfcomponent>